/* $Id: tree_frame.js 2378 2008-11-12 18:24:20Z vadim $ */

function dynamicTree(instanceName, containerId, source) {
  this.containerId = containerId;
  this.instanceName = instanceName;

  this.source = source;
  this.xhr = new httpRequest();

  // clear DIV
  var oDiv = document.getElementById(this.containerId);
  oDiv.innerHTML = '';

  // node data info
  this.nodes = {};

  // preload images
  this.preloadImage('imagePlus',  ICON_PATH + 'i_plus.gif');
  this.preloadImage('imageMinus', ICON_PATH + 'i_minus.gif');
  this.preloadImage('imageEmpty', ICON_PATH + 'px.gif');

  // highlight for the current (selected) node
  this.oHighlightSelected = createElement('DIV', {
                                             'id': 'treeHighlightSelected',
                                             'style.display': 'none'
                                         }, document.body);

  // highlight for the node under the mouse pointer
  this.oHighlightOver = createElement('DIV', {
                                        'id': 'treeHighlightOver',
                                        'style.display': 'none'
                                      }, document.body);

  //  
  this.oSelectedNode = null;

  // load root-level nodes
  this.loadChildren();

  window[instanceName] = this;
}

// preloadImage
dynamicTree.prototype.preloadImage = function(name, src) {
  this[name] = new Image();
  this[name].src = src;
}

// addNode
/*

{nodeData}:    - ,    :
  nodeId (string) -  
  parentNodeId (string) -   
  hasChildren (boolean) -     
  image (string) - 
  name (string) - 
  href (string) -     urlDispatcher  '#subdivision.add(1)'
  action (string!) -    href,     
       js-
  dragEnabled (boolean) -  - (drag-drop)
  expand (boolean) -      (  )
  buttons (object) -  
    - image (string) - 
    - label (string) -  (title, alt) 
    - action (string!) -   (js )
    - href (string) -     urlDispatcher
  parentMustBeLoaded (boolean) -     ,      
  childrenLoaded (boolen) -    ,    loadChildren 
                            (    )
*/
dynamicTree.prototype.addNode = function(nodeData) {

  if (!nodeData) return;

  if (nodeData.parentNodeId) {
    var oParent = this.getChildContainer(nodeData.parentNodeId);
  }
  else {  // add to root node
    oParent = document.getElementById(this.containerId);
  }
  // parent node not found (WTF???)
  if (!oParent) return;

  if (oParent.nodeId && nodeData.parentMustBeLoaded && !this.areChildrenLoaded(oParent.nodeId)) {

    var oPlusMinus = document.getElementById(this.containerId + '_' + oParent.nodeId + '_plusminus');
    oPlusMinus.src = this.imagePlus.src;
    
    this.nodes[oParent.nodeId].hasChildren = true;
    
    return;
  }

  if (this.nodes[oParent.nodeId]) this.nodes[oParent.nodeId].childrenLoaded = true;

  // CREATE NODE

  var oNode = document.createElement('LI');
  oNode.id = this.containerId + '_' + nodeData.nodeId;
  oNode.nodeId = nodeData.nodeId;
  oNode.dragLabel = nodeData.name;
  oNode.treeInstanceName = this.instanceName;

  // +-
  var oPlusMinus = createElement('IMG', {
       id: oNode.id + "_plusminus",
       className: 'plus_minus',
       src: (nodeData.hasChildren ? this.imagePlus.src : this.imageEmpty.src),
       onclick: new Function(this.instanceName + ".toggleNode('" + nodeData.nodeId + "'); ")
   }, oNode);

  // icon
  var oIcon = createElement('IMG', {
    src: ICON_PATH + nodeData.image,
    alt: '',
    width: 16,
    height: 16,
    border: 0
  }, oNode);

  // show id in different style
  if (nodeData.name) {
    nodeData.name = nodeData.name.replace(/^(\d+\.)/, "<span class='node_id'>$1</span>");
  } else {
    return;
  }

  // node name
  if (nodeData.href || nodeData.action) {
    var oCaption = createElement('A', {
      innerHTML: nodeData.name,
      href: nodeData.href
     }, oNode);

    var action = "";
    if (!nodeData.action && nodeData.href && top.urlDispatcher) {
      action = new Function("parent.urlDispatcher.load(this.href.match('#.+$')); " +
                              this.instanceName + ".moveHighlightSelected('" + nodeData.nodeId + "'); " +
                            "return false;");
    }
    else {
      action = new Function(nodeData.action);
    }
//    bindEvent(oCaption, 'click', nodeData.action);
    oCaption.onclick = action;

  } else {
    var oCaption = createElement('DIV', {innerHTML: nodeData.name, 'style.display': 'inline'},oNode);
  }

  // "" 
  //      , .. getElementById() -  
  if (nodeData.buttons) {
    oNode.buttons = createElement('DIV', {
                         'id': oNode.id + '_buttons',
                         'className': 'treeButtons',
                         'style.display': 'none'
                       }, oNode);

    for (var i in nodeData.buttons) {
      var btn = nodeData.buttons[i];

      var oBtnDiv = createElement('DIV', { 'className': 'button' }, oNode.buttons);
      var oBtnLink = false;
      if (btn.link_to != undefined) {
         oBtnLink = createElement('A', { 'href': btn.link_to, 'target': '_blank', 'onclick': function() {return false} }, oBtnDiv);
      }
      createElement('IMG', {
                      'src': ICON_PATH + btn.image,
                      'alt': btn.label,
                      'title': btn.label,
                      'border': 0,
                      'onclick' : (btn.action ? new Function(btn.action) : new Function("parent.urlDispatcher.load('" + btn.href + "'); "))
                    }, (oBtnLink ? oBtnLink : oBtnDiv));
    }

    bindEvent(oNode, 'mouseover', this.showNodeButtons);
    bindEvent(oNode, 'mouseout', this.hideNodeButtons);
  }
  // children container
  var oChildrenContainer = createElement('UL', {
    id: oNode.id + '_children',
    nodeId: nodeData.nodeId,
    'style.display': 'none'
  }, oNode);


  // add drag and drop handlers
  if (nodeData.dragEnabled)  {
    top.dragManager.addDraggable(oCaption, oNode);
    top.dragManager.addDraggable(oIcon, oNode);

    var acceptDropFn = eval(nodeData.acceptDropFn);
    var onDropFn = eval(nodeData.onDropFn);

    top.dragManager.addDroppable(oCaption,
                                 acceptDropFn,
                                 onDropFn,
                                 {name: 'arrowRight', top: 15, left: 4});

    top.dragManager.addDroppable(oIcon,
                                 acceptDropFn,
                                 onDropFn,
                                 {name: 'line', top:16, left: 0});
  }

  // add resulting node to the document
  if (oParent) oParent.appendChild(oNode);

  if (nodeData.expand) { this.toggleNode(nodeData.nodeId); }

  // save settings
  this.nodes[nodeData.nodeId] = nodeData;
}

// update plus-minus (ONLY ON NODE ADDITION/DELETION!)
dynamicTree.prototype.updatePlusMinus = function(nodeId) {

  var oNodeChildren = this.getChildContainer(nodeId);
  var oPlusMinus = document.getElementById(this.containerId + '_' + nodeId + '_plusminus');

  if (!oNodeChildren || !oPlusMinus) return;

  if (this.areChildrenLoaded(nodeId)) { // has children
    if (oNodeChildren.style.display == 'none') {
      oPlusMinus.src = this.imagePlus.src;
    } else {
      oPlusMinus.src = this.imageMinus.src;
    }
  } else {
    oPlusMinus.src = this.imageEmpty.src;
    oNodeChildren.style.display = 'none';
  }
}

// UPDATE NODE
dynamicTree.prototype.updateNode = function(nodeData) {

  var oNode = this.getNode(nodeData.nodeId);
  if (!oNode) return;

  if (nodeData.image) {
    var oIcon = oNode.getElementsByTagName('IMG').item(1);
    oIcon.src = ICON_PATH + nodeData.image;
  }

  var oHref = oNode.getElementsByTagName('A').item(0);
  if (nodeData.href) { oHref.href = nodeData.href; }
  if (nodeData.name) {
    // show id in different style
    nodeData.name = nodeData.name.replace(/^(\d+\.)/, "<span class='node_id'>$1</span>")
    oHref.innerHTML = nodeData.name;
  }

  // save settings
  for (var i in nodeData) {
    this.nodes[nodeData.nodeId][i] = nodeData[i];
  }
}

dynamicTree.prototype.deleteNode = function(nodeId) {
  var oNode = this.getNode(nodeId);
  if (!oNode) { return; }

  // [todo - unbind events?]
  var parent = oNode.parentNode;
  parent.removeChild(oNode);
  this.updatePlusMinus(parent.nodeId);
  oNode = null;
  this.nodes[nodeId] = null;
  this.moveHighlightSelected(this.oSelectedNode.nodeId);
}

/**
  *     (  server-side ,  UI)
  *          
  *
  * @param {String} ID ,   
  * @param {String}    : 'inside'  'below'
  * @param {String} ID
  * @return {Boolean}
  */
dynamicTree.prototype.moveNode = function(nodeId, position, refNodeId) {

  //   
  var oMovedNode = this.getNode(nodeId);
  if (!oMovedNode) { return false; }
  var oOldParent = oMovedNode.parentNode;
  oOldParent.removeChild(oMovedNode);

  //  ?
  if (position == 'inside') { //   refNodeId  
    var oNewParentContainer = this.getChildContainer(refNodeId);
    if (oNewParentContainer.childNodes.length) { // already loaded
      oNewParentContainer.insertBefore(oMovedNode, oNewParentContainer.firstChild);
      if (oNewParentContainer.style.display=='none') { //  
        this.toggleNode(refNodeId);
      }
    }
    else { // node not loaded yet
      this.toggleNode(refNodeId, true);
    }
  }
  else if (position == 'below') { //   refNodeId   
    var oRefNode = this.getNode(refNodeId),
        oNextSibling = oRefNode.nextSibling,
        oNewParentContainer = document.getElementById(oRefNode.parentNode.id);
    //while (oNextSibling.nodeType != 1) {
    //  oNextSibling = oNextSibling.nextSibling;
    //}
  
    //if (oNextSibling) {
      oNewParentContainer.insertBefore(oMovedNode, oNextSibling);
    // }
    // else {
      // oRefNode.parentNode.appendChild(oMovedNode);
    // }
  }
  else { //   position
    return false;
  }

  this.updatePlusMinus(oOldParent.nodeId);
  this.moveHighlightSelected();

  return true;
}

// load children
dynamicTree.prototype.loadChildren = function(nodeId, moreParams) {
  var url = this.source + (nodeId ? nodeId : 'root') + (moreParams ? moreParams : '') + '&rand='+Math.random();
  var on401 = "parent.loginFormShow('document.getElementById(\"treeIframe\").contentWindow."
            + this.instanceName+".loadChildren(\""
            + (nodeId != undefined ? nodeId : '')
            + "\", \""
            + (moreParams != undefined ? moreParams : '')
            + "\")')";
  var nodes = this.xhr.getJson(url, null, {'401': on401});

  if (nodes && nodes.length) {
    for (var i=0; i < nodes.length; i++) {
      if (nodes[i].parentNodeId==undefined) {
        nodes[i].parentNodeId = nodeId;
      }
      this.addNode(nodes[i]);
    }
  }
}


// select node in the tree (+load absent data)
dynamicTree.prototype.selectNode = function(nodeId) {
  var oNode = this.getNode(nodeId);
  if (oNode) { // if node is loaded, show it and all of its parents
    // ... LI[id='siteTree_sub-1'] -> UL[id='siteTree_sub-1_children]' -> LI ...
    oParent = oNode.parentNode;
    while (oParent && oParent.id != this.containerId) { // while root node has been not reached
      if (oParent.tagName=='UL' && oParent.style.display=='none') { // collapsed parent
        this.toggleNode(oParent.nodeId); // open parent
      }
      var oParent = oParent.parentNode;
    }
  }
  else { // if node is not loaded, get path to it and load all parents which haven't been loaded yet
    var path = this.getPath(nodeId);
    if (path && path.length) {
//   
//      path.push(nodeId);

      for (var i=0; i < path.length; i++) {
        var forceLoad = (path[i+1] && this.getNode(path[i+1]) == null);
        this.toggleNode(path[i], forceLoad, true);
      }
      //         ,
      //    
      var oNode = document.getElementById(this.containerId + '_' + nodeId);
      if (!oNode) {
        this.toggleNode(path[path.length - 1], true, true);
      }
    }
    else { return; }
  }

  // highlight node
  this.moveHighlightSelected(nodeId);
}


dynamicTree.prototype.moveHighlightSelected = function (nodeId) {
  if (nodeId) {
    this.oSelectedNode = this.getNode(nodeId);
  }
  if (!this.oSelectedNode) {
    this.oHighlightSelected.style.display = 'none';
    return;
  }
  var top = getOffset(this.oSelectedNode).top;
  if (top < 1) { //      ...
    this.oHighlightSelected.style.display = 'none';
    return;
  }

  // if node is invisible,
  var node = this.oSelectedNode;
  while (node && node.id != this.containerId) {
    if (node.style.display=='none') {
      this.oHighlightSelected.style.display = 'none';
      return;
    }
    node = node.parentNode;
  }

  this.oHighlightSelected.style.top = (top - 3) + 'px';
  this.oHighlightSelected.style.display = '';
}

// get path to the specified node (httpRequest)
dynamicTree.prototype.getPath = function(nodeId) {
  if (!nodeId) return [];
  var url = this.source + nodeId + '&action=get_path&rand=' + Math.random();
  var nodes = this.xhr.getJson(url);
  return nodes;
}

/**
 * Toggle tree node (i.e. show or hide)
 * @param {Object} nodeId
 * @param {Object} forceLoad (used when item dropped on the empty node)
 * @param {Object} onlyIfCollapsed (toggle only if node is collapsed, i.e. open it)
 */
dynamicTree.prototype.toggleNode = function(nodeId, forceLoad, onlyIfCollapsed) {
  // click on the empty node

  if (!forceLoad && document.getElementById(this.containerId + '_' + nodeId + '_plusminus').src==this.imageEmpty.src) {
    return;
  }

  var oNodeChildren = document.getElementById(this.containerId + '_' + nodeId + '_children');

  if (forceLoad) { //       
    oNodeChildren.style.display = 'none';
    oNodeChildren.innerHTML = '';
  }

  var isVisible = (oNodeChildren.style.display != 'none');

  if (onlyIfCollapsed && isVisible) { return; }

  if (!isVisible && !oNodeChildren.childNodes.length) {
    this.loadChildren(nodeId);
  }

  oNodeChildren.style.display = isVisible ? 'none' : '';
  this.updatePlusMinus(nodeId);

  // move highlight
  if (this.oSelectedNode) {
    this.moveHighlightSelected(this.oSelectedNode.nodeId);
  }
  else {
    this.moveHighlightSelected();
  }

/* TODO */
//  if (oNodeChildren.scrollIntoView) oNodeChildren.scrollIntoView();
//  parent.window.scrollTo(0,0);
}

/**
  *       
  * (   -      )
  */
dynamicTree.prototype.showNodeButtons = function(e) {

  if (top.dragManager && top.dragManager.dragInProgress) return;

  // this == node
  // ,      
  var pos = getOffset(this);
  if (!e && window.event) e = window.event;
  if (e.pageY==undefined) {  // IE doesn't have event.pageY property :(
    e.pageY = e.clientY + document.body.scrollTop;
  }
  var distance = Math.abs(e.pageY - pos.top);
  if (distance > 25) return; // , event fired   

  //      , .. getElementById() -  
  if (!this.buttons) return;

  //      
  this.buttons.style.right = -4 + 'px';
  this.buttons.style.display = '';

  if (window.opera) {
    this.buttons.style.top = '1px';
  }

  //    -
  var hl = window[this.treeInstanceName].oHighlightOver;
  hl.style.top = (pos.top - 3) + 'px';
  hl.style.display = '';

  // stop propagation
  if (e && e.stopPropagation) { e.stopPropagation(); }
  else { window.event.cancelBubble = true; }
}

dynamicTree.prototype.hideNodeButtons = function(e) {
  //  
  this.buttons.style.display = 'none';

  //  
  window[this.treeInstanceName].oHighlightOver.style.display = 'none';

  // stop propagation
  if (e && e.stopPropagation) { e.stopPropagation(); }
  else { window.event.cancelBubble = true; }
}

dynamicTree.prototype.getNode = function(nodeId) {
  return document.getElementById(this.containerId + '_' + nodeId);
}

dynamicTree.prototype.getChildContainer = function(nodeId) {
  return document.getElementById(this.containerId + '_' + nodeId + '_children');
}

dynamicTree.prototype.updateNodeHref = function(nodeId, newHref) {
  var oNode = this.getNode(nodeId);
  if (!oNode) return;
  oNode.getElementsByTagName('A')[0].href = newHref;
}

/*
 *  true,    nodeId  
 */
dynamicTree.prototype.areChildrenLoaded = function(nodeId) {
  var oNodeChildren = this.getChildContainer(nodeId);

  if (!oNodeChildren) return false;
  if (oNodeChildren.childNodes.length) return true;

  return false;
}

// ===========================================================================
// DRAG_AND_DROP HANDLERS
// ===========================================================================
function treeSitemapAcceptDrop(e) {
  // dragged item: top.dragManager.draggedInstance.type, .draggedInstance.id
  // being dropped to: top.dragManager.dropTargetObject
  var dragged = top.dragManager.draggedInstance,
      target  = top.dragManager.droppedInstance;

  // rights check
  if (!tree.nodes[target.type + '-' + target.id].dragEnabled) return false;

  // subdivision dropped on subdivision
  if (dragged.type=='sub' && target.type=='sub') return true;
  // subdivision dropped inside the site
  // 'this' refers to the target
  if (dragged.type=='sub' && target.type=='site' && this.tagName=='A') return true;
  // site dropped below the site
  if (dragged.type=='site' && target.type=='site' && this.tagName=='IMG') return true;
  // subclass dropped inside subdivision
  if (dragged.type=='subclass' && target.type=='sub' && this.tagName=='A') return true;

  // message dropped inside subdivision
  if (dragged.type=='message' && target.type=='sub' && this.tagName=='A') {
    //      ?
    var sc = tree.nodes[target.type+'-'+target.id].subclasses;
    for (var i=0; i < sc.length; i++) {
      if (sc[i].classId == dragged.typeNum) { return true; }
    }
  }

  return false;
}


function treeSitemapOnDrop(e) {
  var dragged = top.dragManager.draggedInstance,
      target  = top.dragManager.droppedInstance;
  
  /**
    *     
    */
  if (dragged.type=='message' && target.type=='sub' && this.tagName=='A') {
    //      ?
    var cnt = 0, sc = tree.nodes[target.type+'-'+target.id].subclasses;
    for (var i=0; i < sc.length; i++) {
      if (sc[i].classId == dragged.typeNum) { cnt++; }
    }

    //   --, 
    if (cnt==1) {
      top.moveMessage(dragged.typeNum, dragged.id, sc[0].subclassId);
    }
    else if (cnt > 1) { //     ,   -
      top.showMessageToSubdivisionDialog(dragged.typeNum, dragged.id, target.id);
    }

    return;
  }

  var res = treeGenericDropHandler(top.ADMIN_PATH + 'subdivision/drag_manager_sitemap.php');

  if (res) {
    //       
    if (dragged.type=='subclass' && target.type=='sub' && this.tagName=='A') {
      // change url if 'subclass button' is active (refresh iframe)
      var btnId = 'subclass' + dragged.typeNum + '-' + dragged.id;

      if (top.mainView.toolbar.buttonIsActive(btnId)) {
        top.mainView.refreshIframe();
      }
      else {
        top.mainView.toolbar.removeButton(btnId);
      }
      // update tree nodeData
      var oldSubId = top.dragManager.draggedObject.subdivisionId,
          oldSubData = tree.nodes['sub-' + oldSubId];

      for (var i=0; i < oldSubData.subclasses.length; i++) {
        if (oldSubData.subclasses[i].subclassId==dragged.id) {
          if (i == 0) { // update href if the subclass was first in old subdivision
            var oldSubHref = (oldSubData.subclasses.length == 1)
                               ? '#subclass.add('+oldSubId+')'
                               : '#object.list('+oldSubData.subclasses[i+1].subclassId+')';

            tree.updateNodeHref('sub-'+oldSubId, oldSubHref);
          }
          // remove nodedata
          tree.nodes['sub-'+oldSubId].subclasses.splice(i, 1);
          break; // exit from 'if'
        }
      }

      var newSubData = tree.nodes['sub-'+target.id], firstSubclass;
      if (!newSubData.subclasses || !newSubData.subclasses.length) {
        newSubData.subclasses = [];
        // update href if the subclass is first in new subdivision
        // not checked whether there are children (object.list | subclass.edit)
        firstSubclass = dragged.id;
      }
      else {
        firstSubclass = newSubData.subclasses[0].subclassId;
      }

      tree.updateNodeHref('sub-'+target.id, '#object.list('+ firstSubclass  +')');
      newSubData.subclasses.push({classId: dragged.typeNum, subclassId: dragged.id});
    } // of "      
  } // of "if res"

}


function treeClassAcceptDrop(e) {
  // dragged item: top.dragManager.draggedInstance.type, .draggedInstance.id
  // being dropped to: top.dragManager.dropTargetObject
  var dragged = top.dragManager.draggedInstance,
      target  = top.dragManager.droppedInstance;

  // class dropped inside the class group
  if (dragged.type=='dataclass' && target.type=='group' && this.tagName=='A') return true;
  return false;
}


function treeClassOnDrop(e) {
  var oNode = top.dragManager.draggedObject;
  var oOldParent = oNode.parentNode;

  treeGenericDropHandler('class/drag_manager.php');
  //      ,   
  if (!oOldParent.childNodes.length) oOldParent.parentNode.style.display = 'none';
}

function treeFieldAcceptDrop(e) {
  // dragged item: top.dragManager.draggedInstance.type, .draggedInstance.id
  // being dropped to: top.dragManager.dropTargetObject
  var dragged = top.dragManager.draggedInstance,
      target  = top.dragManager.droppedInstance;

  // rights check
  if (!tree.nodes[target.type + '-' + target.id].dragEnabled) return false;

  var draggedParent = document.getElementById(tree.containerId + '_' + dragged.type + '-' + dragged.id).parentNode;
  var targetParent = document.getElementById(tree.containerId + '_' + target.type + '-' + target.id).parentNode;

  if (draggedParent.id == targetParent.id) {
    // field droped below the field of one class
    if (dragged.type=='field' && target.type=='field' && this.tagName=='IMG') return true;
  }

  return false;
}

function treeFieldOnDrop(e) {
  treeGenericDropHandler('field/drag_manager.php');
}

function treeSystemFieldAcceptDrop(e) {
  // dragged item: top.dragManager.draggedInstance.type, .draggedInstance.id
  // being dropped to: top.dragManager.dropTargetObject
  var dragged = top.dragManager.draggedInstance,
      target  = top.dragManager.droppedInstance;

  // rights check
  if (!tree.nodes[target.type + '-' + target.id].dragEnabled) return false;

  var draggedParent = document.getElementById(tree.containerId + '_' + dragged.type + '-' + dragged.id).parentNode;
  var targetParent = document.getElementById(tree.containerId + '_' + target.type + '-' + target.id).parentNode;

  if (draggedParent.id == targetParent.id) {
    // system field droped below the system field of one class
    if (dragged.type=='systemfield' && target.type=='systemfield' && this.tagName=='IMG') return true;
  }

  return false;
}

function treeSystemFieldOnDrop(e) {
  treeGenericDropHandler('field/drag_manager.php');
}


/**
  * ""  server-side  
  *
  * @param {String}
  * @return {Boolean} success
  */
function treeGenericDropHandler(url) {
  var dragged = top.dragManager.draggedInstance,
      target  = top.dragManager.droppedInstance,
      position = (top.dragManager.dropTargetObject.tagName=='IMG' ? 'below' : 'inside'),
      xhr = new httpRequest(),
      res = xhr.getJson(url,
                          { 'dragged_type': dragged.type,
                            'dragged_id': dragged.id,
                            'position': position,
                            'target_type': target.type,
                            'target_id': target.id
                           } );

  if (res) {
    tree.moveNode(dragged.type+'-'+dragged.id, position, target.type+'-'+target.id);
  }
}
