<?php
/* $Id: s_loadenv.inc.php 2476 2008-12-06 16:56:08Z vadim $ */

function LoadEnv() {
  global $db;
  global $MODULE_VARS;
  global $admin_mode;
  global $catalogue, $sub, $cc;
  global $current_catalogue;
  global $current_sub;
  global $current_cc;
  global $cc_array;
  global $use_multi_sub_class;

  global $system_table_fields, $user_table_mode, $user_table_env, $system_env;
  global $parent_sub_tree, $sub_level_count;

  // cache section
  if ( nc_module_check_by_keyword("cache") ) {
    try {
      $nc_cache_function = nc_cache_function::getObject();
      // query unique string
      $data_string = serialize($MODULE_VARS['auth']);
      // check cached data
      $main_data = array("Catalogue_ID" => $catalogue, "Subdivision_ID" => $sub, "Sub_Class_ID" => $cc);
      $cached_data = $nc_cache_function->read($main_data, $data_string, $MODULE_VARS['cache']['DEFAULT_LIFETIME']);
      if ($cached_data!=-1) {
        // debug info
        $cache_debug_info = "Readed, catalogue[".$catalogue."], sub[".$sub."], cc[".$cc."], Access_ID[-], Lifetime[-], bytes[".strlen($cached_data)."]";
        $nc_cache_function->debugMessage($cache_debug_info, __FILE__, __LINE__);
        // global array values
        list (
          $current_catalogue,
          $current_sub,
          $current_cc,
          $cc_array,
          $system_table_fields,
          $user_table_mode,
          $user_table_env,
          $system_env,
          $parent_sub_tree,
          $sub_level_count
        ) = unserialize($cached_data);
        // return from function
        return;
      }
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_function->errorMessage($e);
    }
  }
   
  LoadSystemSettings();

  // Define $catalogue, $sub and $cc
  switch (true) {
    case !$catalogue:
      $catalogue = $db->get_var("SELECT `Catalogue_ID` FROM `Catalogue` ORDER BY `Priority` LIMIT 1");
      if (!$catalogue) exit;

    case !$sub:
      $sub = $db->get_var("SELECT `Title_Sub_ID` FROM `Catalogue` WHERE `Catalogue_ID` = '".intval($catalogue)."'");
      if (!$sub) exit;

    case !$cc:
      if (!$admin_mode) $checked_only = " AND `Checked` = 1";
      $cc = $db->get_var("SELECT `Sub_Class_ID` FROM `Sub_Class` WHERE `Subdivision_ID` = '".intval($sub)."'".$checked_only." ORDER BY `Priority` LIMIT 1");
  }

  // Load all sub_class id's into array
  if ( !is_array($cc_array) ) {
    $cc_array = array();
    $res = $db->get_results("SELECT `Sub_Class_ID`, `Checked` FROM `Sub_Class` WHERE `Subdivision_ID` = '".intval($sub)."' ORDER BY `Priority`", ARRAY_N);
    if ( !empty($res) ) {
      foreach ($res as $row) {
        if ($row[1]) $cc_array[] = $row[0];
      }
    }
  }

  // Load system table field info
  $res = $db->get_results("SELECT b.`System_Table_Name` AS system_table_name,
    a.`Field_ID` as id,
    a.`Field_Name` as name,
    a.`TypeOfData_ID` as type,
    a.`Inheritance` as inheritance,
    a.`Format` as format
    FROM `Field` AS a, `System_Table` AS b
    WHERE a.`System_Table_ID` = b.`System_Table_ID`
    ORDER BY a.`System_Table_ID`, a.`Priority`", ARRAY_A);
    
    if ( !empty($res) ) {
      foreach ($res AS $row) {
        $system_table_fields[ $row['system_table_name'] ][] = $row;
      }
    }

  // Load environment
  $sub_level_count = 0;

  $current_catalogue = ConvertSystemVars($current_catalogue, "Catalogue");

  $current_sub = InheritSubdivisionEnv($sub);
  $current_cc = InheritSubClassEnv($cc);

  if ($current_sub['Subdivision_ID']==$MODULE_VARS['auth'][USER_MODIFY_SUB] || $current_cc['System_Table_ID']) $user_table_mode = true; else $user_table_mode = false;
  if ($user_table_mode) $user_table_env = LoadSystemTableClass("User");

  $parent_sub_tree[$sub_level_count]["Subdivision_Name"] = $current_catalogue["Catalogue_Name"];
  $parent_sub_tree[$sub_level_count]["Hidden_URL"] = "/";
   
  // cache section
  if ( nc_module_check_by_keyword("cache") && $current_cc['Cache_Access_ID']==1 && is_object($nc_cache_function) ) {
    try {
      $to_save = array($current_catalogue, $current_sub, $current_cc, $cc_array, $system_table_fields, $user_table_mode, $user_table_env, $system_env, $parent_sub_tree, $sub_level_count);
      $bytes = $nc_cache_function->add($main_data, $data_string, serialize($to_save) );
      // debug info
      if ($bytes) {
        $cache_debug_info = "Writed, catalogue[".$catalogue."], sub[".$sub."], cc[".$cc."], Access_ID[-], Lifetime[-], bytes[".$bytes."]";
        $nc_cache_function->debugMessage($cache_debug_info, __FILE__, __LINE__, "ok");
      }
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_function->errorMessage($e);
    }
  }
  
  return;
}

function InheritCatalogueEnv ($catalogue) {
  global $db, $HTTP_FILES_PATH, $system_table_fields;

  $catalogue_env = $db->get_row("SELECT * FROM `Catalogue` WHERE `Catalogue_ID` = '".$catalogue."'", ARRAY_A);

  $catalogue_env = ConvertSystemVars($catalogue_env, "Catalogue");
  
  return $catalogue_env;
}

function InheritSubdivisionEnv ($sub) {
  global $db, $MODULE_VARS, $current_catalogue, $_cache;
  // this global variables change in this function
  global $parent_sub_tree, $sub_level_count;
  //global $system_table_fields, $HTTP_FILES_PATH;

  // cache section
  if ( nc_module_check_by_keyword("cache") ) {
    try {
      $nc_cache_function = nc_cache_function::getObject();
      // init sub_date (from cache or db)
      if (is_array($_cache) && $_cache["sub"][$sub]["Subdivision_ID"] == $sub) {
        $sub_data["Catalogue_ID"]   = $_cache["sub"][$sub]["Catalogue_ID"];
        $sub_data["Subdivision_ID"] = $sub; 
      }
      else {
        $sub_data = $db->get_row("SELECT `Catalogue_ID`, `Subdivision_ID`
        FROM `Subdivision` WHERE `Subdivision_ID` = '".intval($sub)."'", ARRAY_A);  
      }

      // query unique string
      $data_string = serialize($current_catalogue);
      // check cached data
      $cached_data = $nc_cache_function->read($sub_data, $data_string, $MODULE_VARS['cache']['DEFAULT_LIFETIME']);
      if ($cached_data!=-1) {
        // debug info
        $cache_debug_info = "Readed, catalogue[".$sub_data['Catalogue_ID']."], sub[".$sub_data['Subdivision_ID']."], Access_ID[-], Lifetime[-], bytes[".strlen($cached_data)."]";
        $nc_cache_function->debugMessage($cache_debug_info, __FILE__, __LINE__);
        // return cache if not eval flag setted
        $data_array = unserialize($cached_data);
        // global array value
        $parent_sub_tree = $data_array[1];
        // global variable value
        $sub_level_count = $data_array[2];
        // return result
        return $data_array[0];
      }
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_function->errorMessage($e);
    }
  }

  $sub_env = GetSubdivisionByID($sub);
  $sub_env = ConvertSystemVars($sub_env,"Subdivision");

  $sub_level_count = 0;
  $parent_sub_tree[] = "";

  $parent_sub_tree[$sub_level_count] = $sub_env;
  $sub_level_count++;

  $parent_sub = $sub_env["Parent_Sub_ID"];
  
  while ($parent_sub) {
    $parent_sub_env = GetSubdivisionByID($parent_sub);

    $parent_sub_env = ConvertSystemVars($parent_sub_env,"Subdivision");

    if (!$sub_env["Template_ID"]) {
      $sub_env["Template_ID"] = $parent_sub_env["Template_ID"];
      $sub_env["TemplateSettings"] = $parent_sub_env["TemplateSettings"];
    }
    if (!$sub_env["Read_Access_ID"]) $sub_env["Read_Access_ID"] = $parent_sub_env["Read_Access_ID"];
    if (!$sub_env["Write_Access_ID"]) $sub_env["Write_Access_ID"] = $parent_sub_env["Write_Access_ID"];
    if (!$sub_env["Edit_Access_ID"]) $sub_env["Edit_Access_ID"] = $parent_sub_env["Edit_Access_ID"];
    if (!$sub_env["Subscribe_Access_ID"]) $sub_env["Subscribe_Access_ID"] = $parent_sub_env["Subscribe_Access_ID"];
    if ( nc_module_check_by_keyword("cache") ) {
      if (!$sub_env["Cache_Access_ID"]) $sub_env["Cache_Access_ID"] = $parent_sub_env["Cache_Access_ID"];
      if (!$sub_env["Cache_Lifetime"]) $sub_env["Cache_Lifetime"] = $parent_sub_env["Cache_Lifetime"];
    }
    if (!$sub_env["Moderation_ID"]) $sub_env["Moderation_ID"] = $parent_sub_env["Moderation_ID"];

    $parent_sub_tree[$sub_level_count] = $parent_sub_env;
    $sub_env = InheritSystemFields ("Subdivision",$parent_sub_tree[$sub_level_count],$sub_env);

    $sub_level_count++;

    $parent_sub = $parent_sub_env["Parent_Sub_ID"];
  }

  if (!$current_catalogue)
  $catalogue_env = InheritCatalogueEnv ($sub_env["Catalogue_ID"]);
  else
  $catalogue_env = $current_catalogue;

  $sub_env = InheritSystemFields ("Catalogue",$catalogue_env,$sub_env);

  if (!$sub_env["Template_ID"]) {
   $sub_env["Template_ID"] = $catalogue_env["Template_ID"];
   $sub_env["TemplateSettings"] = $catalogue_env["TemplateSettings"];
  }

  if (!$sub_env["Read_Access_ID"]) $sub_env["Read_Access_ID"] = $catalogue_env["Read_Access_ID"];
  if (!$sub_env["Write_Access_ID"]) $sub_env["Write_Access_ID"] = $catalogue_env["Write_Access_ID"];
  if (!$sub_env["Edit_Access_ID"]) $sub_env["Edit_Access_ID"] = $catalogue_env["Edit_Access_ID"];
  if (!$sub_env["Subscribe_Access_ID"]) $sub_env["Subscribe_Access_ID"] = $catalogue_env["Subscribe_Access_ID"];
  if ( nc_module_check_by_keyword("cache") ) {
    if (!$sub_env["Cache_Access_ID"]) $sub_env["Cache_Access_ID"] = $catalogue_env["Cache_Access_ID"];
    if (!$sub_env["Cache_Lifetime"]) $sub_env["Cache_Lifetime"] = $catalogue_env["Cache_Lifetime"];
  }
  if (!$sub_env["Moderation_ID"]) $sub_env["Moderation_ID"] = $catalogue_env["Moderation_ID"];

  $parent_sub_tree[$sub_level_count] = $catalogue_env;
  
  // cache section
  if ( nc_module_check_by_keyword("cache") && $sub_env['Cache_Access_ID']==1 && is_object($nc_cache_function) ) {
    try {
      $to_save = array($sub_env, $parent_sub_tree, $sub_level_count);
      $bytes = $nc_cache_function->add($sub_env, $data_string, serialize($to_save) );
      // debug info
      if ($bytes) {
        $cache_debug_info = "Writed, catalogue[".$sub_env['Catalogue_ID']."], sub[".$sub_env['Subdivision_ID']."], Access_ID[-], Lifetime[-], bytes[".$bytes."]";
        $nc_cache_function->debugMessage($cache_debug_info, __FILE__, __LINE__, "ok");
      }
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_function->errorMessage($e);
    }
  }
  
  return $sub_env;
}

function InheritSystemFields ($system_table_name, $parent_array, $child_array) {
  global $system_table_fields;

  for ($i=0; $i < count($system_table_fields[$system_table_name]); $i++) {
    if (!$system_table_fields[$system_table_name][$i]['inheritance']) continue;
    
    $field_name = $system_table_fields[$system_table_name][$i]['name'];

    if ($child_array[$field_name] == "" || $child_array[$field_name] == NULL) {
      switch ($system_table_fields[$system_table_name][$i]['type']) {
        case 6: // file
          $child_array[$field_name] = $parent_array[$field_name];
          $child_array[$field_name.'_name'] = $parent_array[$field_name.'_name'];
          $child_array[$field_name.'_size'] = $parent_array[$field_name.'_size'];
          $child_array[$field_name.'_type'] = $parent_array[$field_name.'_type'];
          $child_array[$field_name.'_url'] = $parent_array[$field_name.'_url'];
          break;
        case 8: //date
          $child_array[$field_name] = $parent_array[$field_name];
          $child_array[$field_name.'_day'] = $parent_array[$field_name.'_day'];
          $child_array[$field_name.'_month'] = $parent_array[$field_name.'_month'];
          $child_array[$field_name.'_year'] = $parent_array[$field_name.'_year'];
          $child_array[$field_name.'_hours'] = $parent_array[$field_name.'_hours'];
          $child_array[$field_name.'_minutes'] = $parent_array[$field_name.'_minutes'];
          $child_array[$field_name.'_seconds'] = $parent_array[$field_name.'_seconds'];
          break;
        case 10: //multilist
          $child_array[$field_name] = $parent_array[$field_name];
          $child_array[$field_name.'_id'] = $parent_array[$field_name.'_id'];   
        default:
          $child_array[$field_name] = $parent_array[$field_name];
          break;
      }
    }
    else if ( $child_array[$field_name] == 0){ // list,  -   = 0
      if ( $system_table_fields[$system_table_name][$i]['type'] == 4 ) {
        $child_array[$field_name] = $parent_array[$field_name];
        $child_array[$field_name.'_id'] = $parent_array[$field_name.'_id'];
      }
    }
    
  }

  return $child_array;
}

function InheritSubClassEnv ($cc) {
  global $db, $current_sub, $cc_in_sub;


  if ($cc) {
    // check in the $cc_in_sub global array (filled in e404)
    foreach ((array)$cc_in_sub as $row) {
      if ($row["Sub_Class_ID"]==$cc) {
        $cc_env = $row;
        break;
      }
    }

    if (!$cc_env) {
      $cc_env = $db->get_row("SELECT * FROM `Sub_Class` WHERE `Sub_Class_ID` = '".intval($cc)."'",ARRAY_A);
    }


    if ($cc_env) {
      // inherit from Class
      $class_env = $db->get_row("SELECT * FROM `Class` WHERE `Class_ID` = '".intval($cc_env["Class_ID"])."'",ARRAY_A);

      foreach ($class_env AS $key => $val) {
        if ($cc_env[$key]=="") $cc_env[$key] = $val;
      }

      if ($cc_env["NL2BR"] == -1) $cc_env["NL2BR"] = $class_env["NL2BR"];
      if ($cc_env["AllowTags"] == -1) $cc_env["AllowTags"] = $class_env["AllowTags"];
      if ($cc_env["UseCaptcha"] == -1) $cc_env["UseCaptcha"] = $class_env["UseCaptcha"];
    }
  }

  // inherit from Subdivision
  $sub_env = $current_sub;

  if (!$cc_env["Read_Access_ID"]) $cc_env["Read_Access_ID"] = $sub_env["Read_Access_ID"];
  if (!$cc_env["Write_Access_ID"]) $cc_env["Write_Access_ID"] = $sub_env["Write_Access_ID"];
  if (!$cc_env["Edit_Access_ID"]) $cc_env["Edit_Access_ID"] = $sub_env["Edit_Access_ID"];
  if (!$cc_env["Subscribe_Access_ID"]) $cc_env["Subscribe_Access_ID"] = $sub_env["Subscribe_Access_ID"];
  if ( nc_module_check_by_keyword("cache") ) {
    if (!$cc_env["Cache_Access_ID"]) $cc_env["Cache_Access_ID"] = $sub_env["Cache_Access_ID"];
    if (!$cc_env["Cache_Lifetime"]) $cc_env["Cache_Lifetime"] = $sub_env["Cache_Lifetime"];
  }
  if (!$cc_env["Moderation_ID"]) $cc_env["Moderation_ID"] = $sub_env["Moderation_ID"];

  return $cc_env;
}

function ListClassVars ($cc, $user_table_mode) {
  global $db, $MODULE_VARS, $DOMAIN_NAME;
  global $cc_in_sub;
  
  // cache section
  if ( nc_module_check_by_keyword("cache") ) {
    try {
      $nc_cache_function = nc_cache_function::getObject();
      $cc_data = $db->get_row("SELECT `Catalogue_ID`, `Subdivision_ID`, `Sub_Class_ID`
        FROM `Sub_Class` WHERE `Sub_Class_ID` = '".intval($cc)."'", ARRAY_A);
      // make uniqe string concat parameters for this function
      $data_string = $user_table_mode."-".serialize($MODULE_VARS['auth'])."-".$DOMAIN_NAME;
      // check cached data
      $cached_data = $nc_cache_function->read($cc_data, $data_string, $MODULE_VARS['cache']['DEFAULT_LIFETIME']);
      if ($cached_data!=-1) {
        // debug info
        $cache_debug_info = "Readed, catalogue[".$cc_data['Catalogue_ID']."], sub[".$cc_data['Subdivision_ID']."], cc[".$cc_data['Sub_Class_ID']."], Access_ID[-], Lifetime[-], bytes[".strlen($cached_data)."]";
        $nc_cache_function->debugMessage($cache_debug_info, __FILE__, __LINE__);
        // return cache if not eval flag setted
        $Array = unserialize($cached_data);
        return $Array;
      }
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_function->errorMessage($e);
    }
  }
  
  if (!$user_table_mode) {
    $Array = array();

    if ($cc_in_sub) {
      foreach ($cc_in_sub as $row) {
        if ($row["Sub_Class_ID"]==$cc) {
          $Array = $row;
          break;
        }
      }
    }

    if ( empty($Array) ) {
      $db->cache_queries = true;
      $Array = $db->get_row("SELECT * FROM `Sub_Class` WHERE `Sub_Class_ID` = '".intval($cc)."'", ARRAY_A);
      $db->cache_queries = false;
    }

    $class_cond = "`Class_ID` = '".$Array["Class_ID"]."'";
  }
  else {
    $user_table_id = $db->get_var("SELECT `System_Table_ID` FROM `System_Table` WHERE `System_Table_Name` = 'User'");
    $class_cond = "`System_Table_ID` = '".$user_table_id."'";
    $Array["System_Table_ID"] = $user_table_id;
    $Array["Subdivision_ID"] = $MODULE_VARS['auth']['USER_LIST_SUB'];
    $Array["EnglishName"] = $MODULE_VARS['auth']['USER_KEYWORD'];
  }


  //  ,  $cc  
  if (!$Array) {
    echo "<font color='red'>Incorrect \$cc</font>";
    return NULL;
  }

  // inherit from Class
  $class_env = $db->get_row("SELECT * FROM `Class` WHERE ".$class_cond,ARRAY_A);

  if ($class_env['CustomSettingsTemplate']) {
    require_once ($GLOBALS['ADMIN_FOLDER']."array_to_form.inc.php");
    $a2f = new nc_a2f($class_env['CustomSettingsTemplate'], $Array['CustomSettings'], 'CustomSettings');
    $Array["Sub_Class_Settings"] = $a2f->get_values_as_array();
  }

  foreach ($class_env AS $key => $val) {
    if ($Array[$key]=="") $Array[$key] = $val;
  }

  if ($Array["NL2BR"] == -1) $Array["NL2BR"] = $class_env["NL2BR"];
  if ($Array["AllowTags"] == -1) $Array["AllowTags"] = $class_env["AllowTags"];

  if ($Array["Subdivision_ID"]) {
    $parent_sub = $Array["Subdivision_ID"];
    $Domain = $db->get_var("SELECT Catalogue.Domain FROM `Catalogue`
      LEFT JOIN `Subdivision` ON Subdivision.Catalogue_ID = Catalogue.Catalogue_ID
      AND Subdivision.Subdivision_ID = '".$parent_sub."'");
    $HiddenHost = $Domain ? ( strchr($Domain,".") ? $Domain : $Domain.".".$DOMAIN_NAME ) : $DOMAIN_NAME;

    while ($parent_sub) {
      $ParentArray = GetSubdivisionByID($parent_sub);
      
      if ($Array["Subdivision_ID"]==$parent_sub) {
        $Array["Subdivision_Name"] = $ParentArray["Subdivision_Name"];
        $Array["Hidden_URL"] = $ParentArray["Hidden_URL"];
        $Array["Hidden_Host"] = $HiddenHost;
      }

      if (!$Array["Read_Access_ID"]) $Array["Read_Access_ID"] = $ParentArray["Read_Access_ID"];
      if (!$Array["Write_Access_ID"]) $Array["Write_Access_ID"] = $ParentArray["Write_Access_ID"];
      if (!$Array["Edit_Access_ID"]) $Array["Edit_Access_ID"] = $ParentArray["Edit_Access_ID"];
      if (!$Array["Subscribe_Access_ID"]) $Array["Subscribe_Access_ID"] = $ParentArray["Subscribe_Access_ID"];
      if ( nc_module_check_by_keyword("cache") ) {
        if (!$Array["Cache_Access_ID"]) $Array["Cache_Access_ID"] = $ParentArray["Cache_Access_ID"];
        if (!$Array["Cache_Lifetime"]) $Array["Cache_Lifetime"] = $ParentArray["Cache_Lifetime"];
      }
      if (!$Array["Moderation_ID"]) $Array["Moderation_ID"] = $ParentArray["Moderation_ID"];

      $parent_sub = $ParentArray["Parent_Sub_ID"];
    }

    if (!$Array["Catalogue_ID"]) $Array["Catalogue_ID"] = $ParentArray["Catalogue_ID"];
  }

  if ($Array["Catalogue_ID"]==$GLOBALS["catalogue"]) {
    $ParentArray = $GLOBALS["current_catalogue"];
  }
  else {
    $ParentArray = $db->get_row("SELECT * FROM `Catalogue` WHERE `Catalogue_ID` = '".$Array["Catalogue_ID"]."'", ARRAY_A);
  }

  if (!$Array["Read_Access_ID"]) $Array["Read_Access_ID"] = $ParentArray["Read_Access_ID"];
  if (!$Array["Write_Access_ID"]) $Array["Write_Access_ID"] = $ParentArray["Write_Access_ID"];
  if (!$Array["Edit_Access_ID"]) $Array["Edit_Access_ID"] = $ParentArray["Edit_Access_ID"];
  if (!$Array["Subscribe_Access_ID"]) $Array["Subscribe_Access_ID"] = $ParentArray["Subscribe_Access_ID"];
  if ( nc_module_check_by_keyword("cache") ) {
    if (!$Array["Cache_Access_ID"]) $Array["Cache_Access_ID"] = $ParentArray["Cache_Access_ID"];
    if (!$Array["Cache_Lifetime"]) $Array["Cache_Lifetime"] = $ParentArray["Cache_Lifetime"];
  }
  if (!$Array["Moderation_ID"]) $Array["Moderation_ID"] = $ParentArray["Moderation_ID"];
  
  // cache section
  if ( nc_module_check_by_keyword("cache") && $Array['Cache_Access_ID']==1 && is_object($nc_cache_function) ) {
    try {
      $bytes = $nc_cache_function->add($Array, $data_string, serialize($Array) );
      // debug info
      if ($bytes) {
        $cache_debug_info = "Writed, catalogue[".$Array['Catalogue_ID']."], sub[".$Array['Subdivision_ID']."], cc[".$Array['Sub_Class_ID']."], Access_ID[-], Lifetime[-], bytes[".$bytes."]";
        $nc_cache_function->debugMessage($cache_debug_info, __FILE__, __LINE__, "ok");
      }
    }
    catch (Exception $e) {
      // for debug
      $nc_cache_function->errorMessage($e);
    }
  }
  
  return $Array;
}

function InheritTemplateEnv ($template) {
  global $db, $HTTP_FILES_PATH, $INCLUDE_FOLDER, $ADMIN_AUTHTIME;
  global $system_table_fields, $db, $AUTH_USER_ID, $HTTP_HOST, $MODULE_VARS;
  global $admin_mode, $inside_admin, $system_env, $ADMIN_TEMPLATE;

  $template += 0;
  $field_6_types = array();

  for($i=0; $i<count($system_table_fields['Template']); $i++) {
    if ($system_table_fields['Template'][$i]['type']==6) {
       $field_6_types[$i] = $system_table_fields['Template'][$i]['name'];
       $field_string .= "IF(".$system_table_fields['Template'][$i]['name']."<>'',CONCAT('".$HTTP_FILES_PATH.$system_table_fields['Template'][$i]['id']."_',Template_ID,RIGHT(SUBSTRING_INDEX(".$system_table_fields['Template'][$i]['name'].",':',1), LOCATE('.',REVERSE(SUBSTRING_INDEX(".$system_table_fields['Template'][$i]['name'].",':',1))))),'') AS ".$system_table_fields['Template'][$i]['name'].",";
    }
    else {
      $field_string .= $system_table_fields['Template'][$i]['name'].",";
    }
  }

  $template_env = $db->get_row("SELECT ".$field_string."
    `Parent_Template_ID`, `Header`, `Footer`, `Settings`, `CustomSettings`
    FROM `Template`
    WHERE `Template_ID` = '".intval($template)."'", ARRAY_A);

  foreach ($field_6_types AS $tmp_key => $tmp_value) {
    $q = $db->get_row("SELECT `Virt_Name`, `Real_Name`
      FROM `Filetable`
      WHERE `File_Path` = '/t/'
      AND `Message_ID` = '".$template."'
      AND `Field_ID` = '".$system_table_fields['Template'][$tmp_key]['id']."'", ARRAY_N);

    if ($db->num_rows != 0) {
      list($Virt_Name, $Real_Name) = $q;
      $template_env[$tmp_value] = $SUB_FOLDER.$HTTP_FILES_PATH."t/"."h_".$Virt_Name;
      $template_env["{$tmp_value}_url"] = $SUB_FOLDER.$HTTP_FILES_PATH."t/".$Virt_Name;
    }
    else {
      // old-style filesystem
      $template_env[$tmp_value."_url"] = $template_env[$tmp_value];
    }
  }

  $parent_template = $template_env["Parent_Template_ID"];
  
  while ($parent_template) {
    $db->cache_queries = true;
    $parent_template_env = $db->get_row("SELECT {$field_string}Parent_Template_ID,Header,Footer,Settings FROM Template WHERE Template_ID='{$parent_template}'",ARRAY_A);
    $db->cache_queries = false;

    $parent_template = $template_env["Parent_Template_ID"];

    if (!$template_env["Header"])
    {
       $template_env["Header"] = $parent_template_env["Header"];
    }
    if (!$template_env["Footer"])
    {
       $template_env["Footer"] = $parent_template_env["Footer"];
    }

    $template_env["Header"] = str_replace ("%Header",$parent_template_env["Header"], $template_env["Header"]);
    $template_env["Footer"] = str_replace ("%Footer",$parent_template_env["Footer"], $template_env["Footer"]);

    $template_env["Settings"] .= $parent_template_env["Settings"].$template_env["Settings"];

    $template_env = InheritSystemFields("Template",$parent_template_env,$template_env);
    $parent_template = $parent_template_env["Parent_Template_ID"];
  }

  for ($i=0; $i<count($system_table_fields['Template']); $i++) {
    $template_env["Header"] = str_replace("%".$system_table_fields['Template'][$i]['name'],$template_env[$system_table_fields['Template'][$i]['name']],$template_env["Header"]);
    $template_env["Footer"] = str_replace("%".$system_table_fields['Template'][$i]['name'],$template_env[$system_table_fields['Template'][$i]['name']],$template_env["Footer"]);
  }
  
  // add system CSS styles in admin mode
  if ($admin_mode && !$inside_admin && $template_env["Header"]) {
    // addon css
    $template_admin_css = "<link type='text/css' rel='stylesheet' href='".$ADMIN_TEMPLATE."css/admin_pages.css'/>";
    // pattern and replacement for preg_replace()
    switch (true) {
      case preg_match("/\<\s*?\/head\s*?\>/im", $template_env["Header"]):
        $preg_pattern = "/(\<\s*?\/head\s*?\>){1}/im";
        $preg_replacement = $template_admin_css."\n\$1";
      break;
      case preg_match("/\<\s*?html\s*?\>/im", $template_env["Header"]):
        $preg_pattern = "/(\<\s*?html\s*?\>){1}/im";
        $preg_replacement = "\$1\n<head>".$template_admin_css."</head>";
      break;
      default:
        $preg_pattern = "/(\A)/im";
        $preg_replacement = $template_admin_css."\n\$1";
    }
    $template_env["Header"] = preg_replace($preg_pattern, $preg_replacement, $template_env["Header"]);
  }
  
  return $template_env;
}

function LoadSystemTableClass ($SystemTableName) {
  global $db;

  $table_env = $db->get_row("SELECT * FROM `System_Table` AS a, `Class` AS b WHERE a.`System_Table_ID` = b.`System_Table_ID` AND a.`System_Table_Name` = '".$SystemTableName."'", ARRAY_A);

  return $table_env;
}

function LoadSystemSettings () {
  global $db, $system_env;

  $system_env = $db->get_row("SELECT settings.*, field.`Field_Name` AS UserEmail FROM `Settings` AS settings LEFT JOIN `Field` AS field ON settings.`UserEmailField` = field.`Field_ID` ORDER BY `Settings_ID` LIMIT 1", ARRAY_A);

  Header("X-Powered-By: ".$system_env['Powered']);

  return;
}


function GetCatalougeLanguge($catid = 0) {
  global $db, $catalogue, $current_catalogue;
  
  if (!$catalogue) $catalogue = $catid;

  // return current catalogue language
  if (is_array($current_catalogue) && ($current_catalogue["Catalogue_ID"]==$catid || $catid===0)) {
    return $current_catalogue["Language"];
  }

  $res = $db->get_var("SELECT `Language` FROM `Catalogue` WHERE `Catalogue_ID` = '".intval($catalogue)."' LIMIT 1");

  if ($res) {
    return($res);
  }
  else {
    return(false);
  }
}


function LoadModuleEnv () {
  //     vars.inc.php, ..        - include_once!
  global $db, $Catalouge_ID, $catalogue, $MODULE_VARS, $DOMAIN_NAME, $use_gzip_compression;
  global $ADMIN_AUTHTYPE, $PHP_TYPE, $REDIRECT_STATUS, $ADMIN_LANGUAGE, $FILECHMOD, $DIRCHMOD, $SHOW_MYSQL_ERRORS;
  global $SUB_FOLDER, $ROOT_FOLDER, $FILES_FOLDER, $DUMP_FOLDER, $INCLUDE_FOLDER, $TMP_FOLDER, $MODULE_FOLDER, $ADMIN_FOLDER;
  global $HTTP_IMAGES_PATH, $HTTP_ROOT_PATH, $HTTP_FILES_PATH, $HTTP_DUMP_PATH;
  global $ADMIN_PATH, $ADMIN_TEMPLATE, $ADMIN_TEMPLATE_FOLDER;
  global $AUTHORIZE_BY, $AUTHORIZATION_TYPE;

  require_once ($INCLUDE_FOLDER."s_common.inc.php");

  $lang = GetCatalougeLanguge();

  $res = $db->get_results("SELECT `Keyword`, `Parameters` FROM `Module` ORDER BY `Module_ID`", ARRAY_N);
   
	if ( !empty($res) ) {
		foreach ($res as $row) {
			list($keyword,$params) = $row;

			// parse module params
			$query_string = str_replace("\n", "&", $params);
			parse_str($query_string, $MODULE_VARS[$keyword]);

      foreach ($MODULE_VARS[$keyword] AS $var => $val) {
        $MODULE_VARS[$keyword][$var] = trim($val);
      }

			// include language file
			if (is_file($MODULE_FOLDER.$keyword."/".$lang.".lang.php")) {
        require_once($MODULE_FOLDER.$keyword."/".$lang.".lang.php");
			}
      else {
        require_once($MODULE_FOLDER.$keyword."/en.lang.php");
			}

			// include the module itself
			require_once $MODULE_FOLDER.$keyword."/function.inc.php";
		}
	}
}

function ConvertSystemVars ($env_array,$table_name) {
  global $system_table_fields;
  global $HTTP_FILES_PATH, $db;
  
  for ($i=0; $i<count($system_table_fields[$table_name]); $i++) {
    $field_id = $system_table_fields[$table_name][$i]['id'];
    $field_name = $system_table_fields[$table_name][$i]['name'];
    $field_type = $system_table_fields[$table_name][$i]['type'];
    $field_format = $system_table_fields[$table_name][$i]['format'];

    if ($env_array[$field_name]) {
      switch ($field_type) {
        case 4:
          $listname = $db->get_var("SELECT `".$field_format."_Name` FROM `Classificator_".$field_format."` WHERE `".$field_format."_ID` = '".$env_array[$field_name]."'");

          $env_array[$field_name."_id"] = $env_array[$field_name];
          $env_array[$field_name] = $listname;
        break;
        case 6:
          list($filename,$filetype,$filesize) = split(":",$env_array[$field_name]);
          $fileext = substr( $filename, strrpos($filename, ".") );

          $env_array[$field_name."_name"] = $filename;
          $env_array[$field_name."_size"] = $filesize;
          $env_array[$field_name."_type"] = $filetype;

          $q = $db->get_row("SELECT `Virt_Name`, `File_Path` FROM `Filetable` WHERE `Message_ID` = '".$env_array[$table_name."_ID"]."' AND `Field_ID` = '".$field_id."' AND `Real_Name` = '".$filename."' LIMIT 1", ARRAY_N);

          if ($db->num_rows != 0) {
            list($Virt_Name, $File_Path) = $q;
            $env_array[$field_name] = $SUB_FOLDER.$HTTP_FILES_PATH.ltrim($File_Path, '/')."h_".$Virt_Name;
            $env_array[$field_name."_url"] = $SUB_FOLDER.$HTTP_FILES_PATH.ltrim($File_Path, '/').$Virt_Name;
          }
          else {
            $env_array[$field_name] = $SUB_FOLDER.$HTTP_FILES_PATH.$field_id."_".$env_array[$table_name."_ID"].$fileext;
            $env_array[$field_name."_url"] = $env_array[$field_name];
          }

        break;
        case 8:
          $env_array[$field_name."_year"] = substr($env_array[$field_name], 0, 4);
          $env_array[$field_name."_month"] = substr($env_array[$field_name], 5, 2);
          $env_array[$field_name."_day"] = substr($env_array[$field_name], 8, 2);
          $env_array[$field_name."_hours"] = substr($env_array[$field_name], 11, 2);
          $env_array[$field_name."_minutes"] = substr($env_array[$field_name], 14, 2);
          $env_array[$field_name."_seconds"] = substr($env_array[$field_name], 17, 2);
        break;
        case 10:  # Multiselect
          $array_with_id = explode(',' , $env_array[$field_name] );

          if (!$array_with_id[0])                     unset($array_with_id[0]); 
          if (!$array_with_id[count($array_with_id)]) unset($array_with_id[count($array_with_id)]);
          if (empty($array_with_id))                  break;
          
          $listname = $db->get_col("SELECT `".$field_format."_Name` 
                                    FROM `Classificator_".$field_format."` 
                                    WHERE `".$field_format."_ID` IN(".join(',', $array_with_id).")");

          $env_array[$field_name."_id"] = (array)$array_with_id;
          $env_array[$field_name] = (array)$listname;
          break;
          
      }
    }
  }

  return $env_array;
}

function LoadUserEnv () {
  global $db, $AUTH_USER_ID, $current_user;

  // we've got already user info
  if (!$current_user["User_ID"]) {
    $current_user =$db->get_row("SELECT * FROM `User` WHERE `User_ID` = '".$AUTH_USER_ID."'", ARRAY_A);
  }

  $current_user = ConvertSystemVars($current_user, "User");
}

?>